@@ -9,15 +9,24 @@ import os.path
 import sys
 
 import pymysql
+from dateutil import parser as dateparser
 from elasticsearch import Elasticsearch
 from scrapy.exceptions import DropItem
 
+from NewsArticle import NewsArticle
 from .extractor import article_extractor
 from ..config import CrawlerConfig
 
 if sys.version_info[0] < 3:
     ConnectionError = OSError
 
+try:
+    import numpy as np
+    import pandas as pd
+except ModuleNotFoundError:
+    np = None
+    pd = None
+
 
 class HTMLCodeHandling(object):
     """
@@ -86,7 +95,7 @@ class RSSCrawlCompare(object):
         self.cursor = self.conn.cursor()
 
     def process_item(self, item, spider):
-        if spider.name == 'RssCrawler':
+        if spider.name in ['RssCrawler', 'GdeltCrawler']:
             # Search the CurrentVersion table for a version of the article
             try:
                 self.cursor.execute(self.compare_versions, (item['url'],))
@@ -98,14 +107,13 @@ class RSSCrawlCompare(object):
             #   otherwise the result will be overwritten in the buffer
             old_version = self.cursor.fetchone()
 
-            if old_version is not None:
+            if old_version is not None and (datetime.datetime.strptime(
+                    item['download_date'], "%y-%m-%d %H:%M:%S") -
+                                            old_version[3]) \
+                    < datetime.timedelta(hours=self.delta_time):
                 # Compare the two download dates. index 3 of old_version
-                #   corresponds to the download_date attribute in the DB
-                if (datetime.datetime.strptime(
-                        item['download_date'], "%y-%m-%d %H:%M:%S") -
-                        old_version[3]) \
-                        < datetime.timedelta(hours=self.delta_time):
-                    raise DropItem("Article in DB too recent. Not saving.")
+                # corresponds to the download_date attribute in the DB
+                raise DropItem("Article in DB too recent. Not saving.")
 
         return item
 
@@ -304,12 +312,38 @@ class ExtractedInformationStorage(object):
         # clean values
         for key in article:
             value = article[key]
-            if isinstance(value, str):
-                if not value:
-                    article[key] = None
+            if isinstance(value, str) and not value:
+                article[key] = None
 
         return article
 
+    @staticmethod
+    def datestring_to_date(text):
+        if text:
+            return dateparser.parse(text)
+        else:
+            return None
+
+    @staticmethod
+    def convert_to_class(item):
+        news_article = NewsArticle()
+        news_article.authors = item['authors']
+        news_article.date_download = ExtractedInformationStorage.datestring_to_date(item['date_download'])
+        news_article.date_modify = ExtractedInformationStorage.datestring_to_date(item['date_modify'])
+        news_article.date_publish = ExtractedInformationStorage.datestring_to_date(item['date_publish'])
+        news_article.description = item['description']
+        news_article.filename = item['filename']
+        news_article.image_url = item['image_url']
+        news_article.language = item['language']
+        news_article.localpath = item['localpath']
+        news_article.title = item['title']
+        news_article.title_page = item['title_page']
+        news_article.title_rss = item['title_rss']
+        news_article.source_domain = item['source_domain']
+        news_article.text = item['text']
+        news_article.url = item['url']
+        return news_article
+
 
 class InMemoryStorage(ExtractedInformationStorage):
     """
@@ -375,7 +409,7 @@ class JsonFileStorage(ExtractedInformationStorage):
 
         # Write JSON to local file system
         with open(file_path, 'w') as file_:
-            json.dump(ExtractedInformationStorage.extract_relevant_info(item), file_)
+            json.dump(ExtractedInformationStorage.extract_relevant_info(item), file_, ensure_ascii=False)
 
         return item
 
@@ -409,7 +443,7 @@ class ElasticsearchStorage(ExtractedInformationStorage):
                                 client_key=self.database["client_key_path"])
         self.index_current = self.database["index_current"]
         self.index_archive = self.database["index_archive"]
-        self.mapping = {'properties': self.database["mapping"]}
+        self.mapping = self.database["mapping"]
 
         # check connection to Database and set the configuration
 
@@ -425,10 +459,10 @@ class ElasticsearchStorage(ExtractedInformationStorage):
             # check if the necessary indices exist and create them if needed
             if not self.es.indices.exists(self.index_current):
                 self.es.indices.create(index=self.index_current, ignore=[400, 404])
-                self.es.indices.put_mapping(index=self.index_current, doc_type='article', body=self.mapping)
+                self.es.indices.put_mapping(index=self.index_current, doc_type='_doc', body=self.mapping)
             if not self.es.indices.exists(self.index_archive):
                 self.es.indices.create(index=self.index_archive, ignore=[400, 404])
-                self.es.indices.put_mapping(index=self.index_archive, doc_type='article', body=self.mapping)
+                self.es.indices.put_mapping(index=self.index_archive, doc_type='_doc', body=self.mapping)
             self.running = True
 
             # restore previous logging level
@@ -447,12 +481,12 @@ class ElasticsearchStorage(ExtractedInformationStorage):
                 ancestor = None
 
                 # search for previous version
-                request = self.es.search(index=self.index_current, body={'query': {'match': {'url': item['url']}}})
+                request = self.es.search(index=self.index_current, body={'query': {'match': {'url.keyword': item['url']}}})
                 if request['hits']['total'] > 0:
                     # save old version into index_archive
                     old_version = request['hits']['hits'][0]
                     old_version['_source']['descendent'] = True
-                    self.es.index(index=self.index_archive, doc_type='article', body=old_version['_source'])
+                    self.es.index(index=self.index_archive, doc_type='_doc', body=old_version['_source'])
                     version += 1
                     ancestor = old_version['_id']
 
@@ -461,7 +495,7 @@ class ElasticsearchStorage(ExtractedInformationStorage):
                 extracted_info = ExtractedInformationStorage.extract_relevant_info(item)
                 extracted_info['ancestor'] = ancestor
                 extracted_info['version'] = version
-                self.es.index(index=self.index_current, doc_type='article', id=ancestor,
+                self.es.index(index=self.index_current, doc_type='_doc', id=ancestor,
                               body=extracted_info)
 
 
@@ -534,3 +568,90 @@ class DateFilter(object):
                 raise DropItem('DateFilter: %s: Article is too young: %s ' % (item['url'], publish_date))
             else:
                 return item
+
+
+class PandasStorage(ExtractedInformationStorage):
+    """
+    Store meta data a Pandas data frame
+    """
+
+    log = None
+    cfg = None
+    es = None
+    index_current = None
+    index_archive = None
+    mapping = None
+    running = False
+
+    def __init__(self):
+        if np is None:
+            raise ModuleNotFoundError("Using PandasStorage requires numpy and pandas")
+        self.log = logging.getLogger(__name__)
+        self.cfg = CrawlerConfig.get_instance()
+        self.database = self.cfg.section("Pandas")
+
+        df_index = "url"
+        columns = [
+            "source_domain", "title_page", "title_rss", "localpath", "filename",
+            "date_download", "date_modify", "date_publish", "title", "description",
+            "text", "authors", "image_url", "language", 'url'
+        ]
+
+        working_path = self.cfg.section("Files")['working_path']
+        file_name = self.database['file_name']
+        self.full_path = os.path.join(working_path, file_name, '.pickle')
+
+        try:
+            self.df = pd.read_pickle(self.full_path)
+            self.log.info(
+                "Found existing Pandas file with %i rows at %s", len(self.df),
+                self.full_path
+            )
+            for col in columns:
+                if col not in self.df.columns:
+                    raise KeyError(col)
+        except FileNotFoundError:
+            self.df = pd.DataFrame(columns=columns.keys())
+            self.log.info("Created new Pandas file at '%s'", self.full_path)
+            self.df.set_index(df_index, inplace=True, drop=False)
+        except KeyError as e:
+            self.log.error("%s is missing a column.", self.full_path)
+            raise e
+
+    def process_item(self, item, _spider):
+        article = {
+            'authors': item['article_author'],
+            'date_download': item['download_date'],
+            'date_modify': item['modified_date'],
+            'date_publish': item['article_publish_date'],
+            'description': item['article_description'],
+            'filename': item['filename'],
+            'image_url': item['article_image'],
+            'language': item['article_language'],
+            'localpath': item['local_path'],
+            'title': item['article_title'],
+            'title_page': ExtractedInformationStorage.ensure_str(item['html_title']),
+            'title_rss': ExtractedInformationStorage.ensure_str(item['rss_title']),
+            'source_domain':
+            ExtractedInformationStorage.ensure_str(item['source_domain']),
+            'text': item['article_text'],
+            'url': item['url']
+        }
+        self.df.loc[item['url']] = article
+        return item
+
+    def close_spider(self, _spider):
+        """
+        Write out to file
+        """
+        self.df['date_download'] = pd.to_datetime(
+            self.df['date_download'], errors='coerce', infer_datetime_format=True
+        )
+        self.df['date_modify'] = pd.to_datetime(
+            self.df['date_modify'], errors='coerce', infer_datetime_format=True
+        )
+        self.df['date_publish'] = pd.to_datetime(
+            self.df['date_publish'], errors='coerce', infer_datetime_format=True
+        )
+        self.df.to_pickle(self.full_path)
+        self.log.info("Wrote to Pandas to %s", self.full_path)
